#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""package dmm
author  Benoit Dubois
version 0.1
license GPL v3.0+
date    2014-22-05
brief   GUI to handle the 34972A Data Logger via the ethernet interface.
"""

import os
import logging
import tempfile
import numpy as np
from datetime import datetime
from fnmatch import fnmatch
from time import strftime
from PyQt4.QtCore import Qt, pyqtSlot, QObject, QSettings, QThread, QDir, QFileInfo
from PyQt4.QtGui import QMessageBox, QDialog, QFileDialog

import dev34972a as Dev
from utilsben.mjdutils import datetime_to_mjd
from utilsben.datacontainer import TimeSerieContainer
from data_log_form import DmmMainWindow, PreferenceDialog
from dmm.version import __version__
from dmm.dev.constants import ORGANIZATION, APP_NAME, DEF_IP, DEF_PORT, DEF_TIMEOUT


APP_CFG_DIR = QFileInfo(QSettings(ORGANIZATION, APP_NAME).fileName()).absolutePath()


#==============================================================================
# CLASS Dmm34972AGui
#==============================================================================
class Dmm34972AGui(QObject):
    """34972A data logger device Graphical User Interface class.
    GUI over Dev34972A class controller.
    """

    def __init__(self):
        """The constructor.
        :returns: None
        """
        super(Dmm34972AGui, self).__init__()
        settings = QSettings()
        if settings.contains("dev/ip") is False:
            logging.warning("ini file seems empty, " + \
                            "creates one with default values.")
            self._reset_ini()
        ip = settings.value("dev/ip").toString()
        self._data_container = TimeSerieContainer()
        self._acq_start_date = "" # Date @ starting of the acquisition process
        self._tmp_file = ""
        self._ofile = ""
        # Adds ui form
        self._ui = DmmMainWindow()
        # Adds device controller
        self._param_eth = Dev.ParamEth(ip, DEF_PORT, DEF_TIMEOUT)
        self._dev = Dev.Dev34972aWorker(self._param_eth)        
        #self._dev = Stimuli() # For test without the 34972A device
        # Defines behaviour
        self._trigger_ui_actions()
        self._logic_handling()
        # Create producer
        self._prod_thread = QThread()
        self._dev.moveToThread(self._prod_thread)
        self._prod_thread.started.connect(self._dev.start)
        self._dev.finished.connect(self._prod_thread.quit)
        # Shows form
        self._ui.setVisible(True)

    def __del__(self):
        """Stops acquisition properly before exiting application.
        :returns: None
        """
        self._dev.stop()
        self._prod_thread.quit()

    def _logic_handling(self):
        """Defines behaviour of logic.
        :returns: None
        """
        self._dev.new_data.connect(self._write_sample)
        self._dev.new_data.connect(self._update_container)
        self._dev.new_data.connect(self._plot_data)
        # !Only for debug!
        #self._dev.new_data.connect(print_data, Qt.DirectConnection)
        # !Only for debug!
        self._data_container.updated.connect(self._container_updated)
        self._ui.plot_widget.dict.transformations_changed.connect( \
            self._plot_data)
        self._ui.module_widget.channel_state_changed.connect( \
            self._channel_state_change)

    @pyqtSlot()
    def _container_updated(self):
        """Action when data container is updated.
        :returns: None
        """
        if self._data_container.is_empty() is False:
            self._ui.action_save.setEnabled(True)
            self._ui.action_reset.setEnabled(True)
        else:
            self._ui.action_reset.setDisabled(True)

    @pyqtSlot(int)
    def _channel_state_change(self, scan):
        """Updates scan list, the list of channel to scan and update legend
        state of plot widget.
        :param scan: scan number of the channel updated (int)
        :returns: None
        """
        state = self._ui.module_widget.channel(scan).checkState()        
        if state == Qt.Checked:
            self._data_container.add_item(scan)
        else:
            self._data_container.remove_item(scan)

    def set_configuration(self):
        """Sets configuration to real device.
        :returns: None
        """
        for channel in self._ui.module_widget.enabled_channels():
            msg = "CONF:" + Dev.FUNCTION_FULL[channel.func].mnemo \
              + " " + Dev.FUNCTION_FULL[channel.func].list[channel.rang].mnemo
            # Test is used because configuration message syntax needs to be
            # modified if range is in auto mode: User guide indicated that
            # when auto range is used, we must specified "AUTO" for the
            # solution parameter or omit the parameter from the command.
            # But it seems that resolution parameter "AUTO" doesn't work
            # with range parameter "AUTO". So we omit the resolution
            # parameter and thus we must change the message syntax.
            if channel.rang != 0: # ie not "AUTO"
                msg += ", " + Dev.INTGT[channel.intgt].mnemo
            msg += ", (@" + str(channel.id) + ")"
            self._dev.write(msg)

    def set_scanlist(self):
        """Sets scanlist to real device.
        :return: None
        """
        scanlist = ""
        for channel in self._ui.module_widget.enabled_channels():
            scanlist += str(channel.id) + ","
        scanlist = "ROUT:SCAN (@" + scanlist[:-1] + ")"
        try:
            self._dev.write(scanlist)
        except Exception as ex:
            logging.error(str(ex))
            raise

    @pyqtSlot()
    def _reset_ui(self):
        """Clears application: resets user interface to starting state.
        :returns: None
        """
        self._ui.reset()
        self._data_container.reset()

    @pyqtSlot()
    def _quit(self):
        """Quits application.
        :returns: None
        """
        self._dev.stop()
        self._ui.close()

    @pyqtSlot()
    def _start_acq(self):
        """Starts acquisition process.
        :returns: None
        """
        try:
            self._dev.connect()
        except Exception as ex:
            logging.warning("Connection to device failled: %s" % ex)
            return
        self._data_container.clear_data()
        self._ui.module_widget.setDisabled(True)
        self.set_configuration()
        self.set_scanlist()
        self._acq_start_date = strftime("%Y%m%d-%H%M%S")
        self._tmp_file = self._acq_start_date  + ".dat"
        header = self._make_header()
        with open(APP_CFG_DIR + self._tmp_file, 'a', 0) as fd:
            np.savetxt(fname=fd, X=np.empty([0, 0]), header=header)        
        self._prod_thread.start()        

    @pyqtSlot()
    def _stop_acq(self):
        """Stops acquisition request process.
        :returns: None
        """
        self._dev.stop()
        self._dev.close()
        btn_val = self._save_box()
        if btn_val == QMessageBox.Yes: # Stop acquisition and save data
            self._save()
        # In fact saves the last temporary data file, one never knows...
        # But before we move the oldest data files to system temp directory:
        for fd in os.listdir(APP_CFG_DIR):
            if fnmatch(fd, "*.dat.old"):
                os.rename(APP_CFG_DIR + fd, tempfile.tempdir + fd)
        os.rename(APP_CFG_DIR + self._tmp_file, \
                    APP_CFG_DIR + self._tmp_file + ".old")
        self._ui.module_widget.setEnabled(True)

    @pyqtSlot()
    def _save(self):
        """Save data method. Call a file dialog box to choose a filename
        for the data file.
        :returns: True if data are saved else False (bool)
        """
        self._ofile = QFileDialog().getSaveFileName(parent=None, \
                        caption=self.trUtf8("Save data"), \
                        directory=QDir.currentPath(), \
                        filter=self.trUtf8("Data files (*.dat);;Any files (*)"))
        if self._ofile == "":
            return False # Aborts if no file given
        header = self._make_header()
        data = self._data_container.data()
        with open(self._ofile, 'w', 0) as fd:
            np.savetxt(fname=fd, X=data, delimiter='\t', header=header)
        return True

    @pyqtSlot()
    def _save_cfg(self):
        """Saves current device configuration. Call a file dialog box to give
        a filename for the config file.
        :returns: True if config is saved else False (bool)
        """
        cfg_file = QFileDialog().getSaveFileName(None, \
                        self.trUtf8("Save configuration"), \
                        QDir.currentPath(), \
                        self.trUtf8(";;Config files (*.cfg);;Any files (*)"))
        if cfg_file == "":
            return False # Aborts if no file given
        cfg = QSettings(cfg_file, QSettings.IniFormat, self)
        for channel in self._ui.module_widget.itervalues():
            if channel.checkState() == Qt.Checked:
                ch_nb = str(channel.id)
                func = channel.func
                rang = channel.rang
                reso = channel.reso
                cfg.beginGroup(ch_nb)
                cfg.setValue("function", func)
                cfg.setValue("range", rang)
                cfg.setValue("resolution", reso)
                cfg.endGroup()
        return True

    @pyqtSlot()
    def _load_cfg(self):
        """Loads a device configuration. Call a file dialog box to select
        the config file.
        :returns: True if config is loaded else False (bool)
        """
        cfg_file = QFileDialog().getOpenFileName(None, \
                        self.trUtf8("Load configuration"), \
                        QDir.currentPath(), \
                        self.trUtf8("Config files (*.cfg);;Any files (*)"))
        if cfg_file == "":
            return False # Aborts if no file given
        cfg = QSettings(cfg_file, QSettings.IniFormat, self)
        for channel in self._ui.module_widget:
            ch_nb = str(channel.id)
            if cfg.childGroups().contains(ch_nb) is True:
                cfg.beginGroup(ch_nb)
                func = cfg.value("function").toInt()[0]
                rang = cfg.value("range").toInt()[0]
                reso = cfg.value("resolution").toInt()[0]
                cfg.endGroup()
                channel.setState(Qt.Checked)
                channel.set_func(func)
                channel.set_rang(rang)
                channel.set_reso(reso)
            else:
                channel.reset()
        return True

    def _make_header(self):
        """Collects data information (channel, measurement type and parameters)
        to be used as header for a data file.
        :returns: the data header (str)
        """
        now = "Date " + self._acq_start_date + "\n"
        channel_info = ""
        for channel in self._ui.module_widget.channels.itervalues():
            if channel.checkState() == Qt.Checked:
                ch_nb = str(channel.id)
                func = str(channel.param_wdgt.func_cbox.currentText())
                rang = str(channel.param_wdgt.rang_cbox.currentText ())
                reso = str(channel.param_wdgt.reso_cbox.currentText ())
                channel_info += "Channel(" + ch_nb + ");" + func + ";" + \
                  rang + ";" + reso + "\n"
        header = now + channel_info
        return header

    @pyqtSlot(str)
    def _write_sample(self, sample):
        """Writes the current sample(s) in a file.
        :param sample: sample(s) to write (str)
        :returns: None
        """
        now = datetime.utcnow()
        settings = QSettings()
        if settings.value("file/mjd").toBool() is True:
            now = datetime_to_mjd(now)
        sample_list = str(sample).split(',')
        sample_list.insert(0, str(now))
        asample = np.array(sample_list, dtype=float , ndmin=2)
        with open(APP_CFG_DIR + self._tmp_file, 'a', 0) as fd:
            np.savetxt(fname=fd, X=asample, delimiter='\t')

    @pyqtSlot(str)
    def _update_container(self, sample):
        """Adds the current sample(s) to the data container.
        :param sample: sample(s) to add to data container (str)
        :returns: None
        """
        now = datetime_to_mjd(datetime.utcnow())
        self._data_container.add_sample("time", [now])
        enabled_channel_list = self._ui.module_widget.enabled_channels()
        sample_list = str(sample).split(',')
        sample_list.reverse()
        for channel in enabled_channel_list:
            data = np.array([float(sample_list.pop())])
            self._data_container.add_sample(channel.id, data)

    @pyqtSlot()
    def _plot_data(self):
        """Plots data on graph.
        :returns: None
        """
        for key, legend in self._ui.plot_widget.dict.iteritems():
            if legend.checkState() == Qt.Checked:
                scale = legend.transformations.scale
                offset = legend.transformations.offset
                datax = self._data_container.data("time")
                datay = self._data_container.data(key) * scale + offset
                data = np.column_stack((np.transpose(datax),
                                        np.transpose(datay)))
                try:
                    self._ui.plot_widget.curve(key).setData(data)
                except ZeroDivisionError:
                    logging.debug("Divide by zero")
                except ValueError as ex:
                    logging.warning("Exception: %s", str(ex))
                except Exception as ex:
                    logging.warning("Exception: %s", str(ex))
        
    def _reset_ini(self):
        """Resets the ini file with default values.
        :returns: None
        """
        settings = QSettings()
        settings.setValue("dev/ip", DEF_IP)
        settings.setValue("file/mjd", False)

    @pyqtSlot()
    def _preference(self):
        """Displays the preference message box.
        :returns: None
        """
        settings = QSettings()
        dialog = PreferenceDialog(settings.value("dev/ip").toString(),
                                  settings.value("file/mjd").toBool())
        dialog.setParent(None, Qt.Dialog)
        retval = dialog.exec_()
        if retval == QDialog.Accepted:
            settings.setValue("dev/ip", dialog.ip)
            settings.setValue("file/mjd", dialog.mjd)

    @pyqtSlot()
    def _about(self):
        """Displays an about message box.
        :returns: None
        """
        QMessageBox.about(None, self.trUtf8("About " + APP_NAME), \
                          self.trUtf8("<b>" + APP_NAME + "</b> " + \
                          __version__ + "<br>"
                          "GUI dedicated to handle the 34972A device.<br>"
                          "Author Benoit Dubois, benoit.dubois@femto-st.fr.<br>"
                          "Copyright FEMTO Engineering.<br>"
                          "Licensed under the GNU GPL v3.0 or upper."))

    def _save_box(self):
        """Displays a message box requesting confirmation to save the data file.
        Returns the value of the button clicked.
        :returns: the value of the button clicked (int)
        """
        msg_box = QMessageBox()
        msg_box.setWindowTitle(self.trUtf8(APP_NAME))
        msg_box.setText(self.trUtf8("Acquisition stopped"))
        msg_box.setInformativeText(self.trUtf8("Save data?"))
        msg_box.setStandardButtons(QMessageBox.No | QMessageBox.Yes)
        msg_box.setDefaultButton(QMessageBox.Yes)
        btn_val = msg_box.exec_()
        return btn_val

    def _trigger_ui_actions(self):
        """Triggers ui actions: connects actions of ui with real actions.
        :returns: None
        """
        self._ui.action_run.triggered.connect(self._start_acq)
        self._ui.action_stop.triggered.connect(self._stop_acq)
        self._ui.action_save.triggered.connect(self._save)
        self._ui.action_save_cfg.triggered.connect(self._save_cfg)
        self._ui.action_load_cfg.triggered.connect(self._load_cfg)
        self._ui.action_quit.triggered.connect(self._quit)
        self._ui.action_reset.triggered.connect(self._reset_ui)
        self._ui.action_pref.triggered.connect(self._preference)
        # lambda is needed due to un unexplicated bug in action handling:
        # if lambda is not used, no actions work.
        self._ui.action_about.triggered.connect(lambda: self._about())






# -*- coding: utf-8 -*-
# scinstr/version.py

__version__ = "0.4.1"

# 0.4.1  (29/02/2024): Add subpakage 'vnav, 'compressor' and 'phimeter'.
#                      Major update of tctrl subpakage. Also a lot of code
#                      review, bug corrections and code cleaning.
# 0.4.0  (07/04/2023): Separate script files, now in scinstr-bin package,
#                      from driver files.
# 0.3.0  (06/03/2020): - Divide package into subpackage.
#                      - Name of signals use snake case instead of camel case.
# 0.2.0  (13/12/2019): Add support of signal/slot facilities in pure python
#                      using signalslot package.
# 0.1.3  (01/10/2019): Update daq34972a.py to PyQt5.
# 0.1.2  (19/03/2019): Move to PyQt5
# 0.1.1  (14/03/2019): Lot of modifications...
# 0.1.0a0 (18/09/2018): Initial version



# -*- coding: utf-8 -*-

"""package scinstr.daq.labjacq
author    Benoit Dubois
copyright FEMTO Engineering, 2020-2024
licence   GPL 3.0+
brief     Handle Labjack T7(-Pro) device (DAQ board) through USB interface.
details   The USB interface consists of the normal bidirectional control
          endpoint (0 OUT & IN), 3 used bulk endpoints (1 OUT, 2 IN, 3 IN),
          and 1 dummy endpoint (3 OUT). Endpoint 1 consists of a 64 byte OUT
          endpoint (address = 0x01). Endpoint 2 consists of a 64 byte IN
          endpoint (address = 0x82). Endpoint 3 consists of a dummy OUT
          endpoint (address = 0x03) and a 64 byte IN endpoint (address = 0x83).
          Endpoint 3 OUT is not supported, and should never be used.

          All commands should always be sent on Endpoint 1, and the responses
          to commands will always be on Endpoint 2. Endpoint 3 is only used
          to send stream data from the T-series device to the host.
"""

import logging
import time
import usb.core
import usb.util
import pymodbus.exceptions as mdex
from pymodbus.factory import ClientDecoder
from pymodbus.exceptions import ConnectionException
from pymodbus.transaction import ModbusSocketFramer
from pymodbus.client.sync import BaseModbusClient
from scinstr.daq.labjack.constants import T7_VID, T7_PID, T7_USB_TIMEOUT
from scinstr.daq.labjack.t7 import T7


# USB interface parameters
_CONFIG_ID = 1
_INTERFACE_ID = 0   # The interface we use to talk to the device
_ALT_INTF_ID = 0    # The alternate interface use to talk to the device
_OUT_EP_ID = 0      # EP1_OUT @0x01 = 1    -> send command endpoint
_IN_EP_ID = 1       # EP2_IN  @0x82 = 130  -> response endpoint
_OUT_DAT_EP_ID = 2  # EP3_OUT @0x03 = 3    -> !! do not use !!
_IN_DAT_EP_ID = 3   # EP3_IN  @0x83 = 131  -> stream data from T7 to host


# =============================================================================
def get_interfaces():
    """Get available interface of T7 devices.
    Return a list of Usb Device, a named tuple with the following attributes:
    bLength, bDescriptorType, bcdUSB, bDeviceClass, bDeviceSubClass,
    bDeviceProtocol, bMaxPacketSize0, idVendor, idProduct, bcdDevice,
    iManufacturer, iProduct, iSerialNumber, bNumConfigurations, address, bus,
    port_number, port_numbers, speed.
    :returns: list of (usb.core.)Device (list of object)
    """
    interfaces = usb.core.find(idVendor=T7_VID,
                               idProduct=T7_PID,
                               find_all=True)
    return [interface for interface in interfaces]


# =============================================================================
class UsbModbusTcpClient(BaseModbusClient):
    """'TCP' modbus client through USB connection.
    """

    def __init__(self, **kwargs):
        self._ep_in = None
        self._ep_out = None
        self._ep_din = None
        self._ep_dout = None
        self._socket = None
        self._vendor_id = kwargs.get('vendor_id', None)
        self._product_id = kwargs.get('product_id', None)
        self.bus = kwargs.get('bus', None)
        self.address = kwargs.get('address', None)
        self.timeout = kwargs.get('timeout', None)
        super().__init__(ModbusSocketFramer(ClientDecoder(), self))

    def __str__(self):
        """ Builds a string representation of the connection.
        Function is used by pymodbus lib.
        !!!!!!
        Seems that usb driver needs to be recognize as UDP client (?Hack?).
        !!!!!!
        :returns: The string representation (str)
        """
        return "UsbModbusUdpClient(%s:%s)" % (self._vendor_id,
                                              self._product_id)

    def is_socket_open(self):
        """Check whether the underlying socket is open or not.
        :returns: True if socket/serial is open, False otherwise (bool)
        """
        if self._socket is None:
            return False
        return True

    def close(self):
        """ Closes the underlying socket connection
        """
        # Use of explicit release because it seems communication with t7 needs
        # deterministic interface release
        usb.util.release_interface(self._socket, _INTERFACE_ID)
        super().close()
        if self._socket is not None:
            del self._socket
            self._socket = None

    def connect(self):
        """Set an object representing our device (usb.core.Device).
        :param vendor_id: identification number of vendor (int)
        :param product_id: identification number of product (int)
        :returns: True if connection is Ok else returns False (bool)
        """
        if self._socket:
            return True
        # Find our device
        if self.bus is not None and self.address is not None:
            self._socket = usb.core.find(idVendor=self._vendor_id,
                                        idProduct=self._product_id,
                                        bus=self.bus, address=self.address)
        else:
            self._socket = usb.core.find(idVendor=self._vendor_id,
                                        idProduct=self._product_id)
        # Was it found?
        if self._socket is None:
            logging.error("Device not found")
            return False
        # Detach the kernel driver if it is active
        if self._socket.is_kernel_driver_active(0):
            try:
                self._socket.detach_kernel_driver(0)
            except usb.core.USBError as ex:
                logging.critical("Could not detach kernel driver: %r", ex)
                return False
        #
        self._config(config_id=_CONFIG_ID,
                     interface_id=_INTERFACE_ID,
                     alt_interface_id=_ALT_INTF_ID,
                     in_ep_id=_IN_EP_ID,
                     out_ep_id=_OUT_EP_ID,
                     in_dat_ep_id=_IN_DAT_EP_ID,
                     out_dat_ep_id=_OUT_DAT_EP_ID)
        logging.debug("UsbModbusClient connection OK ")
        return self._socket is not None

    def _config(self, config_id, interface_id, alt_interface_id, out_ep_id,
                in_ep_id, out_dat_ep_id, in_dat_ep_id):
        """Configure T7(-Pro) device and get endpoint instance.
        Endpoint Address description:
        - Bits 0..3 Endpoint Number.
        - Bits 4..6 Reserved. Set to Zero
        - Bits 7 Direction 0 = Out, 1 = In (Ignored for Control Endpoints)
        Example: address = 129 = (81)8 = (1000 0001)2
                 -> Direction: In, Number: 1
        :param config_id: ...
        ...
        :returns: None
        """
        try:
            self._socket.set_configuration(config_id)
        except usb.core.USBError as ex:
            logging.critical("USB error during dev configuration: %r", ex)
            raise
        except Exception as ex:
            logging.critical("Could not set interface configuration: %r", ex)
            raise
        # Set interface
        try:
            self._socket.set_interface_altsetting(
                interface=interface_id,
                alternate_setting=alt_interface_id)
        except usb.core.USBError as ex:
            logging.critical("USB error during interface altsetting: %r", ex)
            raise
        except Exception as ex:
            logging.critical("Error during interface altsetting: %r", ex)
            raise
        # Use of explicit release because it seems communication with t7 needs
        # deterministic interface claiming
        usb.util.claim_interface(self._socket, interface_id)
        # Get endpoint instances
        try:
            self._ep_out = self._socket[config_id-1][
                (interface_id, alt_interface_id)][out_ep_id]
            self._ep_in = self._socket[config_id-1][
                (interface_id, alt_interface_id)][in_ep_id]
            self._ep_dout = self._socket[config_id-1][
                (interface_id, alt_interface_id)][out_dat_ep_id]
            self._ep_din = self._socket[config_id-1][
                (interface_id, alt_interface_id)][in_dat_ep_id]
        except usb.core.USBError as ex:
            logging.critical("Could not set endpoint setting: %r", ex)
        logging.debug("USB interface succefuly configured")

    def _send(self, request):
        """ Sends data on the underlying socket
        :param request: The encoded request to send
        :return: The number of bytes written
        """
        if not self._socket:
            raise ConnectionException(self.__str__())
        if request:
            try:
                nb = self._ep_out.write(request, self.timeout)
            except usb.core.USBError as ex:
                logging.error("Could not write data: %r", ex)
                return 0
        return nb

    def _recv(self, size):
        """ Reads data from the underlying descriptor
        :param size: The number of bytes to read
        :return: The bytes read
        """
        if not self._socket:
            raise ConnectionException(self.__str__())

        begin = time.time()
        data = B''

        if size is not None:
            if size < self._ep_din.wMaxPacketSize:
                data = self._ep_in.read(self._ep_din.wMaxPacketSize)
            else:
                while len(data) < size:
                    try:
                        data += self._ep_in.read(self._ep_din.wMaxPacketSize)
                    except usb.core.USBError:
                        pass
                    if not self.timeout \
                       or (time.time() - begin > self.timeout):
                        break
        else:
            while True:
                try:
                    data += self._ep_in.read(self._ep_din.wMaxPacketSize)
                except usb.core.USBError:
                    pass
                if not self.timeout or (time.time() - begin > self.timeout):
                    break

        return data


# =============================================================================
class T7Usb(T7):
    """T7Usb class, provide command/response handling of LabJack T7(-Pro)
    board through USB interface.
    """

    def __init__(self, bus=None, address=None):
        super().__init__()
        self.bus = bus
        self.address = address

    def connect(self):
        """Connect to device.
        :returns: None
        """
        logging.info('Connecting to T7')
        try:
            self._client = UsbModbusTcpClient(vendor_id=T7_VID,
                                              product_id=T7_PID,
                                              bus=self.bus,
                                              address=self.address,
                                              timeout=T7_USB_TIMEOUT)
            is_connected = self._client.connect()
        except mdex.ConnectionException as ex:
            logging.error("Modbus connection error: %r", ex)
            return False
        except Exception as ex:
            logging.error("Unexpected exception: %r", ex)
            return False
        if is_connected is False:
            logging.error('Connection failed')
            return False
        logging.info('Connection --> Ok')
        return True

    def close(self):
        """Close connection with device.
        :returns: None
        """
        try:
            self._client.close()
        except Exception as ex:
            logging.error("Unexpected error %r", ex)
        else:
            logging.info("Connection to T7 closed")


# =============================================================================
if __name__ == "__main__":
    # For "Ctrl+C" works
    import signal
    signal.signal(signal.SIGINT, signal.SIG_DFL)

    # Setup logger
    LOG_FORMAT = '%(asctime)s %(levelname)s %(filename)s (%(lineno)d): ' \
                 + '%(message)s'
    logging.basicConfig(format=LOG_FORMAT, level=logging.INFO)


    I0 = 200e-6

    def pt100_v2t(volt, i0=I0):
        """Temperature from voltage of a Pt100 sensor.
        """
        return (2.604 * volt / i0) - 260.4

    print("Find T7 device(s) with following bus and address:")
    interfaces = get_interfaces()
    for interface in interfaces:
        print("bus: {}, address: {}".format(interface.bus, interface.address))

    _FILE = "t7.dat"

    T7 = T7Usb()
    if not(T7.connect()):
        print("Connection to T7 failed.")
        exit()

    #AINS_LIST = [0, 1, 2, 3]
    #T7.set_ains_resolution(AINS_LIST, [8, 8, 8, 8])

    T7.enable_high_speed_counter(2)

    with open(_FILE, "a") as fd:
        while True:
            #V_LIST = T7.get_ains_voltage(AINS_LIST)
            #V_STRING = ''
            #for V in V_LIST:
            #    V_STRING += str(V) + ';'
            #print(V_STRING)
            # v0 = v_list[1]-v_list[0]
            # v1 = v_list[2]-v_list[3]
            # t0 = pt100_v2t(v0)
            # t1 = pt100_v2t(v1)
            # print(v0, v1,t0, t1)
            # fd.write(v_string + '\n')
            # fd.flush()

            print("Frequency:", T7.get_high_speed_counter_frequency(2))
            time.sleep(1.0)

    T7.close()
    exit()



# -*- coding: utf-8 -*-

"""package dmm
author  Benoit Dubois
version 0.1
license GPL v3.0+
date    2014-25-03
brief   UI to handle the 34972A data logger application.
"""

from PyQt4 import Qt
from PyQt4.QtCore import pyqtSignal, pyqtSlot, QSignalMapper, QEvent
from PyQt4.QtGui import QApplication, QMainWindow, QWidget, QLabel, QAction, \
     QComboBox, QIcon, QVBoxLayout, QCheckBox, QTabWidget, QDialog, QGroupBox, \
     QLineEdit, QDialogButtonBox, QScrollArea, QPushButton

from widgetben import FlowLayout
from widgetben import EPlotWidget

from _34972a_dev import Dev34972aWorker
from dev.constants import DEF_PORT, DEVICE_ID, APP_NAME


#===============================================================================
# Class MyQCheckBox
#===============================================================================
class MyQComboBox(QComboBox):
    """MyQCheckBox class, add generation of enabled change signals.
    """

    enabled_changed = pyqtSignal(bool)

    def __init__(self, *args, **kwargs):
        """Constructor.
        :returns: None
        """
        super(MyQComboBox, self).__init__(*args, **kwargs)

    def changeEvent(self, event):
        """Overloaded method.
        :param event: intercepted event (QEvent)
        :returns: None
        """
        super(MyQComboBox, self).changeEvent(event)
        if event == QEvent.EnabledChange:
            self.enabled_changed.emit(self.isEnabled)


#===============================================================================
# Class PreferenceDialog
#===============================================================================
class PreferenceDialog(QDialog):
    """PreferenceDialog class, generates the ui of the preference form.
    """

    def __init__(self, dev=None, ip="", mjd=False, parent=None):
        """Constructor.
        :param dev: device class instance (object)
        :param ip: IP of 34972A device (str)
        :param mjd: Modified Julian Day flag (bool)
        :param ip: parent object (object)
        :returns: None
        """
        if dev not in DEVICE.values():
            raise KeyError("Device not handled by GUI") 
        super(PreferenceDialog, self).__init__(parent)

        self.setWindowTitle(self.trUtf8("Preferences"))
        # Lays out
        dev_gbox = QGroupBox(self.trUtf8("Interface"))
        self.ip_led = QLineEdit(ip)
        self.ip_led.setInputMask("000.000.000.000;")
        self._check_interface_btn = QPushButton(self.trUtf8("Check"))
        self._check_interface_btn.setToolTip(
            self.trUtf8("Check connection with device"))
        dev_lay = QVBoxLayout()
        dev_lay.addWidget(QLabel(self.trUtf8("IP")))
        dev_lay.addWidget(self.ip_led)
        dev_lay.addWidget(self._check_interface_btn)
        dev_gbox.setLayout(dev_lay)
        file_gbox = QGroupBox(self.trUtf8("Data file"))
        self.mjd_ckbox = QCheckBox()
        if mjd == True:
            self.mjd_ckbox.setCheckState(Qt.Qt.Checked)
        else:
            self.mjd_ckbox.setCheckState(Qt.Qt.Unchecked)
        file_lay = QVBoxLayout()
        file_lay.addWidget(QLabel("Use Modified Julian Day"))
        file_lay.addWidget(self.mjd_ckbox)
        file_gbox.setLayout(file_lay)
        self._btn_box = QDialogButtonBox(QDialogButtonBox.Ok | \
                                         QDialogButtonBox.Cancel)
        main_lay = QVBoxLayout()
        main_lay.addWidget(dev_gbox)
        main_lay.addWidget(file_gbox)
        main_lay.addWidget(self._btn_box)
        self.setLayout(main_lay)
        # Basic logic
        self._btn_box.accepted.connect(self.accept)
        self._btn_box.rejected.connect(self.close)
        self._check_interface_btn.released.connect(dev.check_interface)
        dev.id_checked.connect(self._check_interface)

    @pyqtSlot(bool)
    def _check_interface(self, is_ok):
        """Returns True if interface with device is OK.
        :param is_ok: if True, connection on interface is OK else False (bool)
        :returns: status of interface with device (boolean)
        """
        if is_ok is True:
            self._check_interface_btn.setStyleSheet( \
                "QPushButton { background-color : green; color : yellow; }")
            self._check_interface_btn.setText("OK")
        else:
            self._check_interface_btn.setStyleSheet( \
                "QPushButton { background-color : red; color : blue; }")
            self._check_interface_btn.setText("Error")

    @property
    def ip(self):
        """Getter of the IP value.
        :returns: IP of device (str)
        """
        return self.ip_led.text()

    @property
    def mjd(self):
        """Getter of the mjd parameter selection.
        :returns: MJD state (bool)
        """
        if self.mjd_ckbox.checkState() == Qt.Qt.Checked:
            return True
        else:
            return False


#===============================================================================
# CLASS ParamWdgt
#===============================================================================
class ParamWdgt(QWidget):
    """ParamWdgt class, used to handle the graphical configuration of a
    channel of the 34901 module.
    """

    range_changed = pyqtSignal(int)
    reso_changed = pyqtSignal(int)

    def __init__(self, func_list):
        """Constructor: setup ui.
        :param func_list: list of measurement functions supported by channel
        (list of str)
        :retuns: None
        """
        super(ParamWdgt, self).__init__()
        self.func_cbox = QComboBox()
        self.rang_cbox = QComboBox()
        self.reso_cbox = QComboBox()
        self.intgt_cbox = QComboBox()
        # Lays out
        layout = QVBoxLayout()
        layout.addWidget(self.func_cbox)
        layout.addWidget(self.rang_cbox)
        layout.addWidget(self.reso_cbox)
        layout.addWidget(self.intgt_cbox)
        self.setLayout(layout)
        # Generates signals
        self.reso_cbox.currentIndexChanged.connect(self.reso_changed.emit)
        self.rang_cbox.currentIndexChanged.connect(self.range_changed.emit)
        # UI handling
        self._ui_handling()
        # Inits widgets
        for reso in Dev.RESO:
            self.reso_cbox.addItem(self.trUtf8(reso.caption))
        for intgt in Dev.INTGT:
            self.intgt_cbox.addItem(self.trUtf8(intgt.caption))
        for func in func_list:
            self.func_cbox.addItem(self.trUtf8(func))

    def _ui_handling(self):
        """Basic ui handling. Handles widget enabling/disabling.
        Resolution and integration time value are linked.
        :returns: None
        """
        self.func_cbox.currentIndexChanged.connect(self.set_range_list)
        self.reso_cbox.currentIndexChanged.connect( \
            self.intgt_cbox.setCurrentIndex)
        self.intgt_cbox.currentIndexChanged.connect( \
            self.reso_cbox.setCurrentIndex)
        self.rang_cbox.currentIndexChanged.connect(self._range_handling)

    @pyqtSlot(int)
    def _range_handling(self):
        """Specific range parameter handling.
        Resolution value needs to be disabled if range is set to AUTO mode.
        :returns: None
        """
        if self.rang_cbox.isEnabled() is True:
            if self.rang_cbox.currentIndex() == 0:
                self.reso_cbox.setCurrentIndex(0)
                self.reso_cbox.setDisabled(True)
                self.intgt_cbox.setCurrentIndex(0)
                self.intgt_cbox.setDisabled(True)
            else:
                self.reso_cbox.setEnabled(True)
                self.intgt_cbox.setEnabled(True)
        else:
            self.reso_cbox.setDisabled(True)
            self.intgt_cbox.setDisabled(True)

    def reset(self):
        """Resets the widget.
        :returns: None
        """
        self.func_cbox.setCurrentIndex(0)
        self.rang_cbox.setCurrentIndex(0)
        self.reso_cbox.setCurrentIndex(2)

    @property
    def func(self):
        """Getter for function property.
        :returns: index representing the selected function (int)
        """
        return self.func_cbox.currentIndex()

    @property
    def rang(self):
        """Getter for range property.
        :returns: index representing the selected range (int)
        """
        return self.rang_cbox.currentIndex()

    @property
    def reso(self):
        """Getter for resolution property.
        :returns: index representing the selected resolution (int)
        """
        return self.reso_cbox.currentIndex()

    @property
    def intgt(self):
        """Getter for integration time property.
        :returns: index representing the selected integration time (int)
        """
        return self.intgt_cbox.currentIndex()

    def set_func(self, value):
        """Setter for function property.
        :param value: index representing the selected function (int)
        :returns: None
        """
        self.func_cbox.setCurrentIndex(value)

    def set_rang(self, value):
        """Setter for range property.
        :param value: index representing the selected range (int)
        :returns: None
        """
        self.rang_cbox.setCurrentIndex(value)

    def set_reso(self, value):
        """Setter for resolution property.
        :param value: index representing the selected resolution (int)
        :returns: None
        """
        self.reso_cbox.setCurrentIndex(value)

    def set_intgt(self, value):
        """Setter for integration time property.
        :param value: index representing the selected integration time (int)
        :returns: None
        """
        self.intgt_cbox.setCurrentIndex(value)

    @pyqtSlot(int)
    def set_range_list(self, func_id):
        """Sets a new item list in the range combobox in respect with function
        sets in function combobox widget.
        :param func_id: current function identifier (int)
        :returns: None
        """
        self.rang_cbox.clear()
        range_list = Dev.FUNCTION_FULL[func_id].list
        for item in range_list:
            self.rang_cbox.addItem(self.trUtf8(item.caption))


#===============================================================================
# CLASS ChannelWdgt
#===============================================================================
class ChannelWdgt(QWidget):
    """ChannelWdgt class, used to handle the graphical configuration of a
    channel of the 34901 module.
    """

    state_changed = pyqtSignal(int)
    range_changed = pyqtSignal(int)
    reso_changed = pyqtSignal(int)

    def __init__(self, id_num, func_list):
        """Constructor: setup ui.
        :parma id_num: unique identification number (int)
        :param func_list: list of measurement functions supported by channel
        (list of str)
        :retuns: None
        """
        super(ChannelWdgt, self).__init__()
        self.id = id_num
        self.en_ckbox = QCheckBox(self.trUtf8("Enabled"))
        self.label = QLabel(self.trUtf8("Channel " + str(id_num)))
        self.param_wdgt = ParamWdgt(func_list)
        # Lays out
        layout = QVBoxLayout()
        layout.addWidget(self.label)
        layout.addWidget(self.en_ckbox)
        layout.addWidget(self.param_wdgt)
        self.setLayout(layout)
        # Generates signals
        self.param_wdgt.range_changed.connect(self.range_changed.emit)
        self.param_wdgt.reso_changed.connect(self.reso_changed.emit)
        self.en_ckbox.stateChanged.connect(self.state_changed.emit)
        # UI handling
        self._ui_handling()
        # Inits widget
        self.reset()

    def _ui_handling(self):
        """Basic ui handling. Handles widget enabling/disabling.
        :returns: None
        """
        self.en_ckbox.stateChanged.connect(self.handle_state_change)

    def reset(self):
        """Resets the widget.
        :returns: None
        """
        self.param_wdgt.reset()
        self.setState(Qt.Qt.Unchecked)
        self.setEnabled(False)

    @property
    def func(self):
        """Getter for function property.
        :returns: index representing the selected function (int)
        """
        return self.param_wdgt.func

    @property
    def rang(self):
        """Getter for range property.
        :returns: index representing the selected range (int)
        """
        return self.param_wdgt.rang

    @property
    def reso(self):
        """Getter for resolution property.
        :returns: index representing the selected resolution (int)
        """
        return self.param_wdgt.reso

    @property
    def intgt(self):
        """Getter for integration time property.
        :returns: index representing the selected integration time (int)
        """
        return self.param_wdgt.intgt

    def set_func(self, value):
        """Setter for function property.
        :param value: index representing the selected function (int)
        :returns: None
        """
        self.param_wdgt.set_func(value)

    def set_rang(self, value):
        """Setter for range property.
        :param value: index representing the selected range (int)
        :returns: None
        """
        self.param_wdgt.set_rang(value)

    def set_reso(self, value):
        """Setter for resolution property.
        :param value: index representing the selected resolution (int)
        :returns: None
        """
        self.param_wdgt.set_reso(value)

    def set_intgt(self, value):
        """Setter for integration time property.
        :param value: index representing the selected integration time (int)
        :returns: None
        """
        self.parma_wdgt.set_intgt(value)

    def checkState(self):
        """Redefines method: returns the state of widget ie the state of the
        en_ckbox widget instead of the state of the widget itself, because
        the en_ckbox widget master the state of the widget.
        :returns: the state of the check box (int)
        """
        return self.en_ckbox.checkState()

    def setState(self, state):
        """Sets state property. See checkState().
        :param state: the state of the check box (int)
        """
        self.en_ckbox.setCheckState(state)

    def isEnabled(self):
        """Redefines method: returns the state of widget ie the state of the
        en_ckbox widget instead of the state of the widget itself, because
        the en_ckbox widget master the state of the widget.
        :returns: State of widget (bool)
        """
        return self.en_ckbox.isEnabled()

    def setEnabled(self, flag=True):
        """Redefines method: enables/disables whole widgets except en_ckbox
        widget, because the en_ckbox called this function.
        :param flag: new flag (bool)
        :returns: None
        """
        self.param_wdgt.setEnabled(flag)

    def handle_state_change(self, state):
        """Handles widget in respect with QCheckBox state.
        :param state: new state (int)
        :returns: None
        """
        self.setEnabled(True if state == Qt.Qt.Checked else False)


#===============================================================================
# CLASS Mod34901Wdgt
#===============================================================================
class Mod34901Wdgt(QWidget):
    """W34901Wdgt class, used to handle the graphical configuration of the
    34901 module.
    """

    channel_state_changed = pyqtSignal(int)
    channel_range_changed = pyqtSignal(int)
    channel_reso_changed = pyqtSignal(int)

    def __init__(self, slot=None):
        """Constructor.
        :param slot: slot number, between 1 to 3 (int)
        :returns: None
        """
        super(Mod34901Wdgt, self).__init__()
        self._slot = slot
        self._channels = {}
        for id_key in range(100*slot+1, 100*slot+21):
            self._channels[id_key] = ChannelWdgt(id_key, \
                            [idx.caption for idx in Dev.FUNCTION])
        for id_key in range(100*slot+21, 100*slot+23):
            self._channels[id_key] = ChannelWdgt(id_key, \
                            [idx.caption for idx in Dev.FUNCTION_FULL])
        # Layout
        scroll_layout = QVBoxLayout(self)
        scroll = QScrollArea()
        scroll_layout.addWidget(scroll)
        scroll_wdgt = QWidget(self)
        dyn_layout = FlowLayout(scroll_wdgt)
        for channel in self._channels.itervalues():
            dyn_layout.addWidget(channel)
        scroll.setWidget(scroll_wdgt)
        scroll.setWidgetResizable(True)
        # Channel 21 and 22 are mutualy exclusives
        self._channels[100*slot+21].en_ckbox.stateChanged.connect( \
            lambda: self._exclu_chan(100*slot+21, [100*slot+21, 100*slot+22]))
        self._channels[100*slot+22].en_ckbox.stateChanged.connect( \
            lambda: self._exclu_chan(100*slot+20, [100*slot+21, 100*slot+22]))
        # Generates signals when a channel changes
        ## channel_state_changed()
        state_signal_mapper = QSignalMapper(self)
        for key, channel in self._channels.iteritems():
            state_signal_mapper.setMapping(channel, key)
            channel.state_changed.connect(state_signal_mapper.map)
        state_signal_mapper.mapped.connect(self.channel_state_changed)
        ## channel_range_changed()
        range_signal_mapper = QSignalMapper(self)
        for key, channel in self._channels.iteritems():
            range_signal_mapper.setMapping(channel, key)
            channel.range_changed.connect(range_signal_mapper.map)
        range_signal_mapper.mapped.connect(self.channel_state_changed)
        ## channel_reso_changed()
        reso_signal_mapper = QSignalMapper(self)
        for key, channel in self._channels.iteritems():
            reso_signal_mapper.setMapping(channel, key)
            channel.reso_changed.connect(reso_signal_mapper.map)
        reso_signal_mapper.mapped.connect(self.channel_state_changed)

    def __iter__(self):
        """Iterator: iterates over channels dict.
        """
        return iter(self._channels)

    def itervalues(self):
        """Iterator: iterates over channels dict.
        """
        return iter(self._channels.itervalues())

    def enabled_channels(self):
        """Getter of the list of enabled channel object (list).
        :returns: the list of enabled channel object (list)
        """
        en_list = []
        for channel in self._channels.itervalues():
            if channel.en_ckbox.checkState() == Qt.Qt.Checked:
                en_list.append(channel)
        return en_list

    def reset(self):
        """Resets UI.
        :returns: None
        """
        for channel in self._channels.itervalues():
            channel.reset()

    @property
    def slot(self):
        """Getter of the slot number.
        :returns: the dictionnary of channel object (dict)
        """
        return self._slot

    @property
    def channels(self):
        """Getter of the dictionnary of channel object (dict).
        :returns: the dictionnary of channel object (dict)
        """
        return self._channels

    def channel(self, id_key):
        """Getter of channel widget object (ChannelWdgt).
        :param id_key: key index of channel in the dict channel (str)
        :returns: a channel widget object (ChannelWdgt)
        """
        return self._channels[id_key]

    def _exclu_chan(self, channel_id, channel_id_list):
        """Handles mutual exclusif access of channel 20 and 21 (the only
        channels with capabilities of current measurement) ie ensures that
        only one channel is checked (notes that neither of the channels can
        been checked). Checks that 'channel' is the master over others channels.
        If this channel is checked, the others are unchecked.
        :param channel_id: the id number of the current master channel (int)
        :param channel_id_list: list of id of mutual exclusif channels (list)
        :returns: None
        """
        if self._channels[channel_id].en_ckbox.checkState() == Qt.Qt.Checked:
            channel_id_list.remove(channel_id)
            for ch in channel_id_list:
                self._channels[ch].en_ckbox.setChecked(Qt.Qt.Unchecked)


#===============================================================================
# Class DmmWidget
#===============================================================================
class DmmWidget(QWidget):
    """DmmWidget class, main interface of the UI of the data log form.
    """

    CHANNEL_KEYS_LIST = range(101, 123)
    MODULE_SLOT = 1

    def __init__(self):
        """Constructor.
        :returns: None
        """
        super(DmmWidget, self).__init__()
        # Lays out
        tab = QTabWidget()
        self.module_widget = Mod34901Wdgt(self.MODULE_SLOT)
        self.plot_widget = self._build_plot_widget(self.CHANNEL_KEYS_LIST)
        tab.addTab(self.module_widget, "Configuration")
        tab.addTab(self.plot_widget, "Plot data")
        #tab.addTab(self.dev_widget, "Analyze")
        layout = QVBoxLayout()
        layout.addWidget(tab)
        self.setLayout(layout)
        # Initialization
        for legend in self.plot_widget.dict.legends.itervalues():
            legend.setDisabled(True)
        # Local ui handling
        self.module_widget.channel_state_changed.connect( \
            self._channel_state_change)
        self.plot_widget.dict.state_changed.connect(self._update_plot_list)

    @staticmethod
    def _build_plot_widget(channel_keys):
        """Builds widget dedicated to data plotting.
        :param channel_keys: list of key for each of channel in widget (list)
        :returns: data plotting widget (EPlotWidget)
        """
        plot_widget = EPlotWidget(channel_keys)
        plot_widget.plot.setTitle("Monitoring")
        return plot_widget

    @staticmethod
    def _build_dev_widget():
        """Builds layout dedicated to deviation analyze.
        :returns: deviation plotting widget (AdevPlotWidget)
        """
        #dev_widget = DevPlotWidget()
        #return dev_widget
        pass

    @pyqtSlot(int, int)
    def _update_plot_list(self, scan, state):
        """Updates plot list, the list of channel to plot.
        :param scan: the scan number of the channel updated (int)
        :param state: the state of the channel updated (Qt.State)
        :returns: None
        """
        if state == Qt.Qt.Checked:
            self.plot_widget.hide(scan, False)
        else:
            self.plot_widget.hide(scan, True)

    @pyqtSlot(int)
    def _channel_state_change(self, scan):
        """Updates scan list, the list of channel to scan and update legend
        state of plot widget.
        :param scan: scan number of the channel updated (int)
        :returns: None
        """
        state = self.module_widget.channel(scan).checkState()
        if state == Qt.Qt.Checked:
            self.plot_widget.add_item(scan)
            self.plot_widget.legend(scan).setEnabled(True)
            self.plot_widget.legend(scan).setState(Qt.Qt.Checked)
        else:
            self.plot_widget.legend(scan).setDisabled(True)
            self.plot_widget.legend(scan).setState(Qt.Qt.Unchecked)
            self.plot_widget.remove_item(scan)


#===============================================================================
# Class DmmMainWindow
#===============================================================================
class DmmMainWindow(QMainWindow):
    """MainWindow class, main interface of the UI of the data log form.
    """

    def __init__(self):
        """Constructor.
        :returns: None
        """
        super(DmmMainWindow, self).__init__()
        self.setWindowTitle("Dmmger")
        # Lays out
        self._create_actions()
        self._menu_bar = self.menuBar()
        self._populate_menubar()
        self._tool_bar = self.addToolBar("Tool Bar")
        self._populate_toolbar()
        self._tool_bar.setMovable(True)
        self._tool_bar.setFloatable(False)
        self._tool_bar.setAllowedAreas(Qt.Qt.AllToolBarAreas)
        self._status_bar = self.statusBar()
        self._data_log_wdgt = DmmWidget()
        self.setCentralWidget(self._data_log_wdgt)
        # UI handling
        self._ui_handling()
        # Initialization of UI
        self.reset()

    def reset(self):
        """Resets form.
        :returns: None
        """
        self._data_log_wdgt.module_widget.reset()
        self._data_log_wdgt.plot_widget.reset()
        self.action_run.setDisabled(True)
        self.action_save.setDisabled(True)
        self.action_reset.setDisabled(True)

    @property
    def module_widget(self):
        """Getter of module widget (Mod34901Wdgt).
        :returns: a module widget (Mod34901Wdgt)
        """
        return self._data_log_wdgt.module_widget

    @property
    def plot_widget(self):
        """Getter of plot widget (EPlotWidget).
        :returns: a plot widget (EPlotWidget)
        """
        return self._data_log_wdgt.plot_widget

    @property
    def dev_widget(self):
        """Getter of deviation widget (DevPlotWidget).
        :returns: a deviation widget (DevPlotWidget)
        """
        return self._data_log_wdgt.dev_widget

    def _ui_handling(self):
        """Basic (local) ui handling.
        :returns: None
        """
        # Sets run and stop exclusive actions
        self.action_run.triggered.connect( \
            lambda: self.action_stop.setEnabled(True))
        self.action_stop.triggered.connect( \
            lambda: self.action_run.setEnabled(True))
        self.action_run.triggered.connect( \
            lambda: self.action_run.setEnabled(False))
        self.action_stop.triggered.connect( \
            lambda: self.action_stop.setEnabled(False))
        self.action_stop.setEnabled(False)
        # Handles UI when the state of a channel changes
        self._data_log_wdgt.module_widget.channel_state_changed.connect( \
            self._channel_state_change)

    @pyqtSlot()
    def _channel_state_change(self):
        """Handles ui when the state of channels changes:
        updates the state of the run button of bars.
        :returns: None
        """
        if len(self._data_log_wdgt.module_widget.enabled_channels()) == 0:
            # If no channel selected, do not allow start of acquisition
            self.action_run.setDisabled(True)
        else:
            # If channel(s) selected, allow start of acquisition
            self.action_run.setEnabled(True)

    def _create_actions(self):
        """Creates actions used with bar widgets.
        :returns: None
        """
        self.action_save = QAction(QIcon.fromTheme("document-save"), \
                                   self.trUtf8("&Save"), self)
        self.action_save.setStatusTip(self.trUtf8("Save data"))
        self.action_save.setShortcut('Ctrl+S')

        self.action_save_cfg = QAction(self.trUtf8("Save &Config"), self)
        self.action_save_cfg.setStatusTip(self.trUtf8( \
                                        "Save the configuration for later"))
        self.action_save_cfg.setShortcut('Ctrl+C')

        self.action_load_cfg = QAction(self.trUtf8("&Load Config"), self)
        self.action_load_cfg.setStatusTip(self.trUtf8( \
                                        "Load a previous configuration"))
        self.action_load_cfg.setShortcut('Ctrl+L')

        self.action_quit = QAction(QIcon.fromTheme("application-exit"), \
                                   self.trUtf8("&Quit"), self)
        self.action_quit.setStatusTip(self.trUtf8("Exit application"))
        self.action_quit.setShortcut('Ctrl+Q')

        self.action_reset = QAction(QIcon.fromTheme("edit-clear"), \
                                    self.trUtf8("R&eset"), self)
        self.action_reset.setStatusTip(self.trUtf8("Reset the configuration"))
        self.action_reset.setShortcut('Ctrl+E')

        self.action_pref = QAction(QIcon.fromTheme("preferences-system"), \
                                    self.trUtf8("&Preferences"), self)
        self.action_pref.setStatusTip(self.trUtf8( \
                                    "Open preference dialog form"))
        self.action_pref.setShortcut('Ctrl+P')

        self.action_run = QAction(QIcon.fromTheme("system-run"), \
                                  self.trUtf8("&Run"), self)
        self.action_run.setStatusTip(self.trUtf8("Start acquisition"))
        self.action_run.setShortcut('Ctrl+R')

        self.action_stop = QAction(QIcon.fromTheme("process-stop"), \
                                   self.trUtf8("S&top"), self)
        self.action_stop.setStatusTip(self.trUtf8("Stop acquisition"))
        self.action_stop.setShortcut('Ctrl+T')

        self.action_about = QAction(QIcon.fromTheme("help-about"), \
                                    self.trUtf8("A&bout"), self)
        self.action_about.setStatusTip(self.trUtf8("About " + APP_NAME))
        self.action_about.setShortcut('Ctrl+B')

    def _populate_menubar(self):
        """Populates the menu bar of the UI
        :returns: None
        """
        self._menu_bar.menu_file = self._menu_bar.addMenu(self.trUtf8("&File"))
        self._menu_bar.menu_edit = self._menu_bar.addMenu(self.trUtf8("&Edit"))
        self._menu_bar.menu_process = self._menu_bar.addMenu( \
                                        self.trUtf8("&Process"))
        self._menu_bar.menu_help = self._menu_bar.addMenu(self.trUtf8("&Help"))
        self._menu_bar.menu_file.addAction(self.action_save)
        self._menu_bar.menu_file.addSeparator()
        self._menu_bar.menu_file.addAction(self.action_save_cfg)
        self._menu_bar.menu_file.addAction(self.action_load_cfg)
        self._menu_bar.menu_file.addSeparator()
        self._menu_bar.menu_file.addAction(self.action_quit)
        self._menu_bar.menu_edit.addAction(self.action_pref)
        self._menu_bar.menu_process.addAction(self.action_run)
        self._menu_bar.menu_process.addAction(self.action_stop)
        self._menu_bar.menu_process.addAction(self.action_reset)
        self._menu_bar.menu_help.addAction(self.action_about)

    def _populate_toolbar(self):
        """Populates the tool bar of the UI
        :returns: None
        """
        self._tool_bar.addAction(self.action_run)
        self._tool_bar.addAction(self.action_stop)
        self._tool_bar.addAction(self.action_reset)
        self._tool_bar.addAction(self.action_save)


#===============================================================================
def display_ui():
    """Displays the main UI.
    """
    import sys

    def print_slot(arg):
        """Print 'arg' to standard output.
        :param arg: data to display (any)
        :returns: None
        """
        print "ui.print_slot.arg;", arg

    app = QApplication(sys.argv)
    #ui = ChannelWdgt(101, ("DC Voltage", "AC Voltage", "Resistance"))
    #ui = Mod34901Wdgt(1)
    #ui = DmmWidget()
    ui = DmmMainWindow()
    ui.module_widget.channel_state_changed.connect(print_slot)
    ui.show()
    sys.exit(app.exec_())


#===============================================================================
if __name__ == "__main__":
    display_ui()



# -*- coding: utf-8 -*-

# Default Tektronix DMM 4050 parameters
DMM4050_ID = "TEKTRONIX,DMM4050"
DMM4050_PORT = 3490
DMM4050_TIMEOUT = 3.0

# Default Keysight DMM 3446(0|1)A parameters
DMM34461A_ID = "Technologies,3446"
DMM34461A_PORT = 5025
DMM34461A_TIMEOUT = 3.0
DMM34461A_VID = 0x2a8d
DMM34461A_PID = 0x1601

